/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.


     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/

#if defined(_MSC_VER)
#pragma warning ( disable : 4786 )
#endif

#include <fstream>
#include "otbSOMMap.h"
#include "otbSOMClassifier.h"
#include "itkImageToListAdaptor.h"

int otbSOMClassifierNew(int argc, char* argv[] )
{ 
  try
    {
      typedef double                         InputPixelType;
      typedef int                            LabelPixelType;
      const   unsigned int        	     Dimension = 2;
      
      typedef itk::RGBPixel<InputPixelType> PixelType;
      typedef itk::Statistics::EuclideanDistance<PixelType> DistanceType;
      typedef otb::SOMMap<PixelType,DistanceType,Dimension> SOMMapType;
      typedef otb::Image<PixelType,Dimension> InputImageType;
      typedef itk::Statistics::ImageToListAdaptor< InputImageType > SampleType;
      typedef otb::SOMClassifier<SampleType,SOMMapType,LabelPixelType> ClassifierType;    
      
      ClassifierType::Pointer classifier = ClassifierType::New() ;
    }
  catch( itk::ExceptionObject & err ) 
    { 
      std::cout << "Exception itk::ExceptionObject levee !" << std::endl; 
      std::cout << err << std::endl; 
      return EXIT_FAILURE;
    } 
  catch( ... ) 
    { 
      std::cout << "Unknown exception !" << std::endl; 
      return EXIT_FAILURE;
    } 
 
  return EXIT_SUCCESS;
}







